from types import SimpleNamespace


class Namespace(SimpleNamespace):
    def get(self, key):
        for k, v in vars(self).items():
            if key == k or key == v:
                return v
        raise KeyError('Unknown key value.')

    def get_key(self, value, default=None):
        for k, v in vars(self).items():
            if value == v:
                return k
        return default


DST_ADDR = 0xfffe
SRC_ADDR = 0x8000
PROTO = 0xf0
RESERVED = 0x00


Devices = Namespace(
    NO_DEVICE=0,
    BX6QL=0x0066,
    BX6Q1=0x0166,
    BX6Q2=0x0266,
    BX6Q3=0x0366,
    BX6Q2L=0x0466,
    BX6Q3L=0x0566
)
"""List of supported controllers:

    * BX6QL
    * BX6Q1
    * BX6Q2
    * BX6Q3
    * BX6Q2L
    * BX6Q3L
"""

Commands = Namespace(
    UPDATE=(0x01, 0xa7, 0x00),
    DELETE=(0x01, 0xa7, 0x01),
    PING=(0x01, 0xa2, 0x00)
)

Errors = Namespace(
    ERR_NO=0x00,
    ERR_OUTOFGROUP=0x01,
    ERR_CMD=0x02,
    ERR_BUSY=0x03,
    ERR_MEMORYVOLUME=0x04,
    ERR_CHECKSUM=0x05,
    ERR_FILENOTEXIST=0x06,
    ERR_FLASH=0x07,
    ERR_FILE_DOWNLOAD=0x08,
    ERR_FILE_NAME=0x09,
    ERR_FILE_TYPE=0x0A,
    ERR_FILE_CRC16=0x0B,
    ERR_FONT_NOT_EXIST=0x0C,
    ERR_FIRMWARE_TYPE=0x0D,
    ERR_DATE_TIME_FORMAT=0x0E,
    ERR_FILE_EXIST=0x0F,
    ERR_FILE_BLOCK_NUM=0x10,
    ERR_CONTROLLER_TYPE=0x11,
    ERR_SCREEN_PARA=0x12,
    ERR_CONTROLLER_ID=0x13
)

RunModes = Namespace(
    CYCLE=0x00,
    LAST_PAGE_STATIC=0x01,
    CYCLE_TIMEOUT=0x02,
    CYCLE_TIMEOUT_LAST=0x03,
    ONCE=0x04
)
"""Dynamic area run modes:

    ==================   =====================================================
     Mode                 Description
    ==================   =====================================================
    CYCLE                Infinite loop
    LAST_PAGE_STATIC     Freezes on last page
    CYCLE_TIMEOUT        Looped, but limited by timeout [1]
    CYCLE_TIMEOUT_LAST   Looped, limited by timeout and freezes on last page
    ONCE                 Played once
    ==================   =====================================================

        [1] `timeout` parameter in :class:`~.dynamic_area.DynamicArea` constructor
"""

DisplayModes = Namespace(
    RANDOM=0x00,
    STATIC=0x01,
    DIRECT_SHOW=0x02,
    MOVE_LEFT=0x03,
    CONTINUOUS_MOVE_LEFT=0x04,
    MOVE_UP=0x05,
    CONTINUOUS_MOVE_UP=0x06,
    FLICKER=0x07,
    SNOWING=0x08,
    BUBBLING=0x09,
    MIDDLE_MOVE_OUT=0x0a,
    HORIZONTAL_CROSS_MOVE=0x0b,
    VERTICAL_CROSS_MOVE=0x0c,
    UP_AND_DOWN_CROSS_MOVE=0x0d,
    SCROLL_CLOSED=0x0e,
    SCROLL_OPENED=0x0f,
    LEFT_STRETCH=0x10,
    RIGHT_STRETCH=0x11,
    UP_STRETCH=0x12,
    DOWN_STRETCH=0x13,
    LEFT_LASER=0x14,
    RIGHT_LASER=0x15,
    UP_LASER=0x16,
    DOWN_LASER=0x17,
    CROSS_CURTAIN_LEFT_AND_RIGHT=0x18,
    CROSS_CURTAIN_UP_AND_DOWN=0x19,
    LEFT_PULL_DISPERSEDLY=0x1a,
    HORIZONTAL_BLINDS=0x1b,
    VERTICAL_BLINDS=0x1c,
    CURTAIN_LEFT=0x1d,
    CURTAIN_RIGHT=0x1e,
    CURTAIN_UP=0x1f,
    CURATIN_DOWN=0x20,
    TO_CENTER_FROM_LEFT_AND_RIGHT=0x21,
    SPLIT_TO_LEFT_AND_RIGHT=0x22,
    TO_CENTER_FROM_UP_AND_DOWN=0x23,
    SPLIT_TO_UP_AND_DOWN=0x24,
    MOVE_RIGHT=0x25,
    CONTINUOUS_MOVE_RIGHT=0x26,
    MOVE_DOWN=0x27,
    CONTINUOUS_MOVE_DOWN=0x28
)
"""Pages 'flipping' modes."""
